#!/usr/bin/env python2

import sys
import re
import string
import argparse

#globals
ip4Nets = {}
fileMatch = {}
fileNoMatch = {}
printFileNames = False
files = []
returnCode = 1

def ip4Int(ip4Addr):
	"ip4Int: convert IPv4 address to integer"
	octets = map(int, ip4Addr.split('.'))
	return((octets[0] << 24) + (octets[1] << 16) + (octets[2] << 8) + octets[3])

def ip4PrefixInt(ip4Prefix):
	"ip4PrefixInt: convert IPv4 prefix to integer"
	return(2**(32-int(ip4Prefix)))

def ip4Match(ip4String):
	"ip4Match: return true if string matches IPv4 address"
	return(re.search(r"^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", ip4String))

def ip4Load(ipString):
	"ip4Load: append networks to the ip4Nets dict"
	ip4Net = ipString.split('/')
	if ip4Match(ip4Net[0]):
		ipNet = ip4Int(ip4Net[0])
	else:
		print >> sys.stderr, "Invalid network: " + ipString
		return
	
	if ip4Match(ip4Net[1]):
		ipMask = ip4Int(ip4Net[1])
	else:
		if (0 < int(ip4Net[1]) <= 32):
			ipMask = ((2**32 - 1) << (32 - int(ip4Net[1])))
		else:
			print >> sys.stderr, "Invalid network: " + ipString
			return

	#after all validation, load the (base) network and the netmask
	ip4Nets[(ipNet & ipMask)] = ipMask

def ip4Grep(grepfile, fileName):
	"ip4Grep: grep for lines matching network patterns"
	for line in grepfile:
		#list of ip addresses - same as ip4Match but no anchors
		words = re.findall(r"(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)", line)
		lineMatched = False
		if (words):
			for word in words:
				ipInt = ip4Int(word)
				for net, mask in ip4Nets.iteritems():
					if ((ipInt & mask) == net):
						lineMatched = True
						fileMatch[fileName] += 1
						if (not args.q and not args.c and not args.v):
							if (printFileNames):
								print "%s:" % fileName,
							print line,
						break
				if (lineMatched):
					break 

		#non-matching lines
		else:
			fileNoMatch[fileName] += 1
			if (args.v and not args.q and not args.c):
				if (printFileNames):
					print "%s:" % fileName,
				print line,

def printCount(fileName):
	"printCount: print a count of matching (or non-matching) lines"
	if (args.q):
		return
	if (printFileNames):
		print "%s:" % fileName,
	if (args.v):
		print "%i" % fileNoMatch[fileName]
	else:
		print "%i" % fileMatch[fileName]
	return

#parse and set default options
parser = argparse.ArgumentParser(
	formatter_class=argparse.RawDescriptionHelpFormatter,
	description='Search text for networks.',
	epilog='''Network patterns can be specified as:
  network/prefix     example: 192.168.1.0/24
  network/mask       example: 172.16.16.0/255.255.240.0''')
parser.add_argument('-c', action='store_true', help='display a count of matching lines')
parser.add_argument('-q', action='store_true', help="don't print any output")
parser.add_argument('-f', metavar='file', help='read network patterns from "file", one per line')
parser.add_argument('-v', action='store_true', help='invert matching (print/count non-matching lines)')
parser.add_argument('network', nargs='?')
parser.add_argument('files', nargs='*')
args = parser.parse_args()

#trigger help if not give a network patter or pattern file
if not (args.network or args.f):
	parser.parse_args(['-h'])
	exit(255)

#if the first argument appears to be a network pattern
if re.match(r"^[0-9.]+/[0-9.]+$", args.network):
	ip4Load(args.network)
else:
	#it must be a file name
	files.append(args.network)
	
#complete the list of files
files += args.files

#load patterns from file if given -f
if (args.f):
	with open(args.f, 'r') as patFile:
		for line in patFile:
			if re.match('^[ 	]*#', line): #comments
				continue
			if re.match('^[ 	]*$', line): #blank lines
				continue
			ip4Load(line.strip())
		patFile.close()

if not (ip4Nets):
	print >> sys.stderr, "No valid network patters!"
	exit(254)

if (len(args.files) > 1):
	printFileNames = True

#if given a list of files
if (files):
	for fileName in files:
		with open(fileName, 'r') as inFile:
			fileMatch[fileName] = 0
			fileNoMatch[fileName] = 0
			ip4Grep(inFile, fileName)
			if (args.c):
				printCount(fileName)
		inFile.close()
#else use stdin
else:
	fileMatch["stdin"] = 0
	fileNoMatch["stdin"] = 0
	ip4Grep(sys.stdin, "stdin")
	if (args.c):
		printCount("stdin")

#set the return code based on matching type
if (args.v):
	for fileName, count in fileNoMatch.iteritems():
		if (count > 0):
			#at least one line did not match
			returnCode = 0
			break
else:
	for fileName, count in fileMatch.iteritems():
		if (count > 0):
			#at least one line matched
			returnCode = 0
			break

exit(returnCode)
